@@ -1,9 +1,7 @@
-# You are free to use/modify/redistribute this program as long as original authorship credit is given and you
-# in no way impinge on its free distribution.
-
-#' The MARX function
+#' Mixed causal-noncausal autoregressions with exogenous regressors.
 #'
-#' This interface-based function allows you to perform model selection for MARX models based on information criteria.
+#' @title The MARX function
+#' @description This interface-based function allows you to perform model selection for MARX models based on information criteria.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p_max Maximum number of autoregressive parameters (leads + lags) to be included.
@@ -193,9 +191,8 @@ marx <- function(y,x,p_max,sig_level,p_C,p_NC){
   cat(' ', "\n")
 }
 
-#' The regressor matrix function
-#'
-#' This function allows you to create a regressor matrix.
+#' @title The regressor matrix function
+#' @description This function allows you to create a regressor matrix.
 #' @param y   Data vector of time series observations.
 #' @param x   Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p   Number of autoregressive terms to be included.
@@ -204,7 +201,7 @@ marx <- function(y,x,p_max,sig_level,p_C,p_NC){
 #' @author Sean Telg
 #' @export
 #' @examples
-#' data <- sim.marx(c('t',3,0),c('t',1,1),100,0.5,0.4,0.3)
+#' data <- sim.marx(c('t',3,1),c('t',1,1),100,0.5,0.4,0.3)
 #' regressor.matrix(data$y, data$x, 2)
 
 regressor.matrix <- function(y,x,p){
@@ -255,9 +252,8 @@ regressor.matrix <- function(y,x,p){
   return(matrix = Z)
 }
 
-#' The ARX estimation by OLS function
-#'
-#' This function allows you to estimate ARX models by ordinary least squares (OLS).
+#' @title The ARX estimation by OLS function
+#' @description This function allows you to estimate ARX models by ordinary least squares (OLS).
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p Number of autoregressive terms to be included.
@@ -275,7 +271,7 @@ regressor.matrix <- function(y,x,p){
 #' @author Sean Telg
 #' @export
 #' @examples
-#' data <- sim.marx(c('t',3,0),c('t',1,1),100,0.5,0.4,0.3)
+#' data <- sim.marx(c('t',3,1),c('t',1,1),100,0.5,0.4,0.3)
 #' arx.ls(data$y,data$x,2)
 
 arx.ls <- function(y,x,p){
@@ -345,9 +341,8 @@ arx.ls <- function(y,x,p){
   return(list(coefficients = B, coef.auto = B_auto, coef.exo = B_x, mse = Cov, residuals = U, loglikelihood = Loglik, fitted.values = FV, df = df,vcov=vcov))
 }
 
-#' The estimation of the MARX model by t-MLE function
-#'
-#' This function allows you to estimate the MARX model by t-MLE.
+#' @title The estimation of the MARX model by t-MLE function
+#' @description This function allows you to estimate the MARX model by t-MLE.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p_C Number of lags.
@@ -362,10 +357,11 @@ arx.ls <- function(y,x,p){
 #' @return \item{scale}{Estimated scale parameter.}
 #' @return \item{df}{Estimated degrees of freedom.}
 #' @return \item{residuals}{Residuals.}
+#' @return \item{se.dist}{Standard errors of the distributional parameters.}
 #' @author Sean Telg
 #' @export
 #' @examples
-#' data <- sim.marx(c('t',3,0),c('t',3,1),100,0.5,0.4,0.3)
+#' data <- sim.marx(c('t',3,1),c('t',3,1),100,0.5,0.4,0.3)
 #' marx.t(data$y,data$x,1,1)
 
 marx.t <- function(y,x,p_C,p_NC,params0){
@@ -474,10 +470,10 @@ marx.t <- function(y,x,p_C,p_NC,params0){
       else if (p_C == 0 && p_NC == 0){
         B_NC  <- 0
         B_C   <- 0
-        B_x   <- PARAMS[(p_C + 3):(p_C + 3 + numcol)]
-        IC    <- PARAMS[(p_C + numcol + 4)]
-        sig   <- PARAMS[(p_C + numcol + 5)]
-        df    <- PARAMS[(p_C + numcol + 6)]
+        B_x   <- PARAMS[(p_C + 3):(p_C + 2 + numcol)]
+        IC    <- PARAMS[(p_C + numcol + 3)]
+        sig   <- PARAMS[(p_C + numcol + 4)]
+        df    <- PARAMS[(p_C + numcol + 5)]
       }
   }
   else{
@@ -580,12 +576,15 @@ marx.t <- function(y,x,p_C,p_NC,params0){
 
   }
 
-  return(list(coef.c = B_C, coef.nc = B_NC, coef.exo = B_x, coef.int = IC, scale = sig,df = df,residuals = E))
+  se <- sqrt(diag(solve(optimization_results$hessian)))
+  se.dist <- se[(length(se)-1):length(se)]
+  se.dist <- rev(se.dist)
+
+  return(list(coef.c = B_C, coef.nc = B_NC, coef.exo = B_x, coef.int = IC, scale = sig,df = df,residuals = E, se.dist = se.dist))
 }
 
-#' Asymptotic inference for the MARX function
-#'
-#' This function allows you to calculate standard errors and confidence intervals for parameters of the MARX model.
+#' @title Asymptotic inference for the MARX function
+#' @description This function allows you to calculate standard errors and confidence intervals for parameters of the MARX model.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param B_C Estimated causal parameters of the MARX.
@@ -632,11 +631,10 @@ inference <- function(y,x,B_C,B_NC,B_x,IC,sig,df,sig_level){
     p_NC = 0
   }
 
-  T = length(y) - p_C - p_NC
+  obser = length(y) - p_C - p_NC
 
   regressor_C <- regressor.matrix(y,"not",p_C)
-  regressand_C <- y[(p_C+1):length(y)]
-  regressand_C <- fBasics::vec(regressand_C)
+  regressand_C <- fBasics::vec(y[(p_C+1):length(y)])
 
   if (p_C == 1){
     regressor_C <- fBasics::vec(regressor_C)
@@ -644,22 +642,23 @@ inference <- function(y,x,B_C,B_NC,B_x,IC,sig,df,sig_level){
 
   regressor_NC  <- regressor.matrix(fBasics::vec(rev(y)),"not",p_NC)
 
-  if (p_NC == 1){
-    regressor_NC <- fBasics::vec(regressor_NC)
-  }
-
   if (p_NC > 1){
-    for (i in 1:length(regressor_NC[1,])){
-      regressor_NC[,i] <- rev(regressor_NC[,i])
+    for(i in 1:NCOL(regressor_NC)){
+      regressor_NC[,i] <- fBasics::vec(rev(regressor_NC[,i]))
     }
   }
 
   if (p_NC == 1){
-    regressor_NC <- fBasics::vec(regressor_NC)
+    regressor_NC <- fBasics::vec(rev(fBasics::vec(regressor_NC)))
   }
 
-  regressand_NC <- y[1:(length(y)-p_NC)]
-  regressand_NC <- fBasics::vec(regressand_NC)
+  #if (p_NC > 1){
+  #  for (i in 1:length(regressor_NC[1,])){
+  #    regressor_NC[,i] <- rev(regressor_NC[,i])
+  #  }
+  #}
+
+  regressand_NC <- fBasics::vec(y[1:(length(y)-p_NC)])
 
   ## Causal part:
 
@@ -672,14 +671,14 @@ inference <- function(y,x,B_C,B_NC,B_x,IC,sig,df,sig_level){
     }
 
     U_regressor_C <- regressor.matrix(U,"not",p_C)
-    Gam_C <- (1/(T-(p_C+p_NC)))* (t(U_regressor_C) %*% U_regressor_C)
-    Cov_C <- (df+3)/(df+1)*sig^2*fBasics::inv(Gam_C)
+    Gam_C <- (t(U_regressor_C) %*% U_regressor_C)/obser
+    Cov_C <- (df+3)/(df+1)*sig^2*solve(Gam_C)
 
     std_c <- matrix(data=NA,nrow=length(Cov_C[,1]), ncol=1)
     BC <- matrix(data=NA,nrow=length(Cov_C[,1]), ncol=2)
 
     for(i in 1:length(Cov_C[,1])){
-      std_c[i] = sqrt(Cov_C[i,i]/T)
+      std_c[i] = sqrt(Cov_C[i,i]/obser)
       BC[i,] = c((B_C[i] - stats::qnorm(1-(sig_level/2))*std_c[i]),(B_C[i] + stats::qnorm(1-(sig_level/2))*std_c[i]))
     }
   }
@@ -701,22 +700,29 @@ inference <- function(y,x,B_C,B_NC,B_x,IC,sig,df,sig_level){
 
     V_regressor_NC <- regressor.matrix(fBasics::vec(rev(V)),"not",p_NC)
 
+
+    if (p_NC > 1){
+      for(i in 1:NCOL(V_regressor_NC)){
+        V_regressor_NC[,i] <- fBasics::vec(rev(V_regressor_NC[,i]))
+      }
+    }
+
     if (p_NC == 1){
-      V_regressor_NC <- fBasics::vec(V_regressor_NC)
+      V_regressor_NC <- fBasics::vec(rev(fBasics::vec(V_regressor_NC)))
     }
 
-    for (i in 1:length(V_regressor_NC[1,])){
-      V_regressor_NC[,i] <- rev(V_regressor_NC[,i])
-    }
+    #for (i in 1:length(V_regressor_NC[1,])){
+    #  V_regressor_NC[,i] <- rev(V_regressor_NC[,i])
+    #}
 
-    Gam_NC <- (1/(T-(p_C + p_NC)))* (t(V_regressor_NC) %*% V_regressor_NC)
-    Cov_NC <- (df+3)/(df+1)*sig^2*fBasics::inv(Gam_NC)
+    Gam_NC <- (1/obser)* (t(V_regressor_NC) %*% V_regressor_NC)
+    Cov_NC <- (df+3)/(df+1)*sig^2*solve(Gam_NC)
 
     std_nc <- matrix(data=NA,nrow=length(Cov_NC[,1]), ncol=1)
     BNC <- matrix(data=NA,nrow=length(Cov_NC[,1]), ncol=2)
 
     for(i in 1:length(Cov_NC[,1])){
-      std_nc[i] = sqrt(Cov_NC[i,i]/T)
+      std_nc[i] = sqrt(Cov_NC[i,i]/obser)
       BNC[i,] = c((B_NC[i] - stats::qnorm(1-(sig_level/2))*std_nc[i]),(B_NC[i] + stats::qnorm(1-(sig_level/2))*std_nc[i]))
     }
   }
@@ -728,19 +734,19 @@ inference <- function(y,x,B_C,B_NC,B_x,IC,sig,df,sig_level){
   ## Intercept
 
   Cov_IC <- (df+3)/(df+1)*sig^2
-  std_ic <- sqrt(Cov_IC/T)
+  std_ic <- sqrt(Cov_IC/obser)
   BIC <- c((IC - stats::qnorm(1-(sig_level/2))*std_ic),(IC + stats::qnorm(1-(sig_level/2))*std_ic))
 
   ## Coefficients of exogenous variables
   if (length(x) > 1){
-      Gam_x <- (1/(T-(p_C + p_NC))) * (t(x) %*% x)
-      Cov_x <- (df+3)/(df+1)*sig^2*fBasics::inv(Gam_x)
+      Gam_x <- (1/obser) * (t(x) %*% x)
+      Cov_x <- (df+3)/(df+1)*sig^2*solve(Gam_x)
 
       std_x <- matrix(data=NA,nrow=length(Cov_x[,1]), ncol=1)
       Bx <- matrix(data=NA,nrow=length(Cov_x[,1]), ncol=2)
 
     for(i in 1:length(Cov_x[,1])){
-      std_x[i] = sqrt(Cov_x[i,i]/T)
+      std_x[i] = sqrt(Cov_x[i,i]/obser)
       Bx[i,] = c((B_x[i] - stats::qnorm(1-(sig_level/2))*std_x[i]),(B_x[i] + stats::qnorm(1-(sig_level/2))*std_x[i]))
     }
   }
@@ -752,9 +758,8 @@ inference <- function(y,x,B_C,B_NC,B_x,IC,sig,df,sig_level){
   return(list(CI.c = BC, CI.nc = BNC, CI.exo = Bx, CI.int = BIC, se.c = std_c, se.nc = std_nc, se.exo = std_x, se.int = std_ic))
 }
 
-#' The value of the t-log-likelihood for MARX function
-#'
-#' This function allows you to determine the value of the t-log-likelihood for the MARX model.
+#' @title The value of the t-log-likelihood for MARX function
+#' @description This function allows you to determine the value of the t-log-likelihood for the MARX model.
 #' @param params List of parameters.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
@@ -922,9 +927,8 @@ ll.max <- function(params,y,x,p_C,p_NC){
   return(neg.loglikelihood = loglik_eval)
 }
 
-#' The lag-lead model selection for MARX function
-#'
-#' This function allows you to determine the MARX model (for p = r + s) that maximizes the t-log-likelihood.
+#' @title The lag-lead model selection for MARX function
+#' @description This function allows you to determine the MARX model (for p = r + s) that maximizes the t-log-likelihood.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p_pseudo Number of autoregressive terms to be included in the pseudo-causal model.
@@ -936,7 +940,7 @@ ll.max <- function(params,y,x,p_C,p_NC){
 #' @author Sean Telg
 #' @export
 #' @examples
-#' data <- sim.marx(c('t',3,0), c('t',3,0),100,0.5,0.4,0.3)
+#' data <- sim.marx(c('t',3,1), c('t',3,1),100,0.5,0.4,0.3)
 #' selection.lag.lead(data$y,data$x,2)
 
 selection.lag.lead <- function(y,x,p_pseudo){
@@ -977,9 +981,8 @@ selection.lag.lead <- function(y,x,p_pseudo){
   return(list(p.C = p_C, p.NC = p_NC,loglikelihood = rev(loglik)))
 }
 
-#' The model selection for pseudo-ARX function
-#'
-#' This function allows you to calculate AIC, BIC, HQ for pseudo-ARX models.
+#' @title The model selection for pseudo-ARX function
+#' @description This function allows you to calculate AIC, BIC, HQ for pseudo-ARX models.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p_max Maximum number of autoregressive terms to be included.
@@ -1038,9 +1041,8 @@ selection.lag <- function(y,x,p_max){
   return(list(bic = bic_vec, aic = aic_vec, hq = hq_vec))
 }
 
-#' The Bayesian/Schwarz information criterion (BIC) function
-#'
-#' This function allows you to calculate the Bayesian/Schwarz information criteria (BIC) for ARX models.
+#' @title The Bayesian/Schwarz information criterion (BIC) function
+#' @description This function allows you to calculate the Bayesian/Schwarz information criteria (BIC) for ARX models.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p_max Maximum number of autoregressive terms to be included.
@@ -1089,9 +1091,8 @@ bic <- function(y,x,p_max){
 
 }
 
-#' The Akaike information criterion (AIC) function
-#'
-#' This function allows you to calculate the Akaike information criteria (AIC) for ARX models.
+#' @title The Akaike information criterion (AIC) function
+#' @description This function allows you to calculate the Akaike information criteria (AIC) for ARX models.
 #' @param y Data vector of time series observations.
 #' @param x Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p_max Maximum number of autoregressive terms to be included.
@@ -1140,9 +1141,8 @@ aic <- function(y,x,p_max){
 
 }
 
-#' The Hannan-Quinn (HQ) information criterion function
-#'
-#' This function allows you to calculate the Hannan-Quinn (HQ) information criteria for ARX models.
+#' @title The Hannan-Quinn (HQ) information criterion function
+#' @description This function allows you to calculate the Hannan-Quinn (HQ) information criteria for ARX models.
 #' @param y       Data vector of time series observations.
 #' @param x       Matrix of data (every column represents one time series). Specify NULL or "not" if not wanted.
 #' @param p_max   Maximum number of autoregressive terms to be included.
@@ -1191,9 +1191,8 @@ hq <- function(y,x,p_max){
 
 }
 
-#' The simulation of MARX processes
-#'
-#' This function allows you to simulate MARX processes based on different underlying distribution.
+#' @title The simulation of MARX processes
+#' @description This function allows you to simulate MARX processes based on different underlying distribution.
 #' @param dist.eps vector containing the error distribution and its parameters (options: t, normal, stable).
 #' @param dist.x   vector containing the distribution of x and its parameters (options: t, normal, stable). Specify NULL or "not" if not wanted.
 #' @param obs      Number of observations for simulated process.
@@ -1203,7 +1202,7 @@ hq <- function(y,x,p_max){
 #' @keywords simulation
 #' @return \item{y}{Simulated data y.}
 #' @return \item{x}{Simulated data x (exogenous variable).}
-#' @author         Sean Telg
+#' @author Sean Telg
 #' @export
 #' @examples
 #' dist.eps <- c('t',1,1) ## t-distributed errors with 1 degree of freedom and scale parameter 1
@@ -1214,7 +1213,7 @@ hq <- function(y,x,p_max){
 #' exo_par <- 0.5
 #' sim.marx(dist.eps,dist.x,obs,c_par,nc_par,exo_par) ## Simulates a MARX(2,1,1) process
 
-sim.marx <- function(dist.eps, dist.x, obs,c_par,nc_par, exo_par){
+sim.marx <- function(dist.eps, dist.x, obs, c_par, nc_par, exo_par){
 
   if (is.null(dist.x)){
     exo_par = 0
@@ -1229,7 +1228,7 @@ sim.marx <- function(dist.eps, dist.x, obs,c_par,nc_par, exo_par){
   M.star <- 500   #0.05*M
 
   if (dist.eps[1] == 't'){
-    eps <- stats::rt((3*(obs+200)), df = as.numeric(dist.eps[2]), ncp = as.numeric(dist.eps[3]))
+    eps <- as.numeric(dist.eps[3])*(stats::rt((3*(obs+200)), df = as.numeric(dist.eps[2]), ncp = 0))
   } else if (dist.eps[1] == 'cauchy'){
     eps <- stats::rcauchy((3*(obs+200)), location= as.numeric(dist.eps[2]), scale=as.numeric(dist.eps[3]))
   } else if (dist.eps[1] == 'normal') {
@@ -1239,7 +1238,7 @@ sim.marx <- function(dist.eps, dist.x, obs,c_par,nc_par, exo_par){
   }
 
   if (dist.x[1] == 't'){
-    x <- stats::rt((3*(M+200)), df = as.numeric(dist.x[2]), ncp = as.numeric(dist.x[3]))
+    x <- as.numeric(dist.x[3])*(stats::rt((3*(M+200)), df = as.numeric(dist.x[2]), ncp = 0))
   } else if (dist.x[1] == 'cauchy'){
     x <- stats::rcauchy((3*(M+200)), location= as.numeric(dist.x[2]), scale=as.numeric(dist.x[3]))
   } else if (dist.x[1] == 'normal') {
@@ -1334,9 +1333,302 @@ sim.marx <- function(dist.eps, dist.x, obs,c_par,nc_par, exo_par){
   return(list(y = y, x = exo_var))
 }
 
-#' The pseudo-causal model function
-#'
-#' This function allows you to estimate pseudo-causal ARX models by OLS (compatible with most functions in lm() class).
+#' @title Companion form function
+#' @description   This function allows you to compute a companion form matrix in order to check the stability of causal and noncausal part of the ARX model.
+#' @param pol     Coefficient vector. If polynomial is 1 - ax - bx^2, coefficient vector is c(a, b).
+#' @keywords stability, stationarity
+#' @return \item{C}{Companion matrix C.}
+#' @author Sean Telg
+#' @export
+#' @examples
+#' pol <- c(0.3,0.4)
+#' C <- companion.form(pol)
+
+companion.form <- function(pol){
+  r <- length(pol)
+  b <- cbind(diag((r-1)),rep(0,(r-1)))
+  C <- rbind(t(pol),b)
+
+  return(C)
+}
+
+#' @title Coefficients of the moving average representation function
+#' @description   This function allows you to invert a polynomial (either the causal or the noncausal one) and output the corresponding coefficients of the moving average representation.
+#' @param pol     Coefficient vector. If polynomial is 1 - ax - bx^2, coefficient vector is c(a, b).
+#' @param M       Truncation value M (how many MA coefficients should be computed?).
+#' @keywords stability, stationarity
+#' @return \item{psi}{Vector containing coefficients of the moving average representation.}
+#' @author Sean Telg
+#' @export
+#' @examples
+#' pol <- c(0.3,0.4)
+#' psi <- companion.form(pol)
+
+compute.MA <- function(pol,M){
+  r <- length(pol)
+  str <- c(rep(0,(r-1)),1)
+  psi <- c(1)
+
+  for (j in 1:M){
+    psi[j+1] <- t(pol) %*% rev(str)
+
+    if (r > 1){
+      str <- c(str[2:length(str)],psi[j+1])
+    }
+    else{
+      str <- psi[j+1]
+    }
+  }
+  return(psi)
+}
+
+#' @title Forecasting function for the MARX model
+#' @description   This function allows you to forecast with the mixed causal-noncausal model with possibly exogenous regressors.
+#' @param y       Data vector y.
+#' @param X       (optional) Matrix with data (column represent a series).
+#' @param p_C     Number of lags (causal order).
+#' @param p_NC    Number of leads (noncausal order).
+#' @param X.for   (optional) Matrix with forecasted values for X (column represents series).
+#' @param h       Forecast horizon h.
+#' @param M       (optional) Truncation value M for MA representation. Default value: 50.
+#' @param N       (optional) Number of simulations to forecast noncausal component. Default: 10,000.
+#' @keywords forecasting
+#' @return \item{y.for}{Vector containing forecasted values for y.}
+#' @author Sean Telg
+#' @export
+#' @examples
+#' ## Forecasting MAR(0,1) model 4-periods ahead for lnbev (from dataset)
+#' data <- MARX::dataset[,2]
+#' y.for <- forecast.marx(y=data, p_C=0, p_NC=1, h=4, M=50, N=1000)
+
+forecast.marx <- function(y,X,p_C,p_NC,X.for,h,M,N){
+
+  set.seed(9999)
+  if (missing(X) == TRUE){
+    X = NULL
+  }
+
+  if (missing(N) == TRUE){
+    N = 10000
+  }
+
+  object <- mixed(y,X,p_C,p_NC)
+  obs <- length(y)
+
+  ## Check whether there are exogenous variables and whether truncation M is known
+
+  if (missing(X.for) == TRUE && missing(M) == TRUE){
+    X.for = NULL
+    M = 50
+  }
+  else if(missing(X.for) == TRUE && missing(M) == FALSE){
+    X.for = NULL
+    M = M
+  }
+  else if(missing(X.for) == FALSE && missing(M) == TRUE){
+    if (NCOL(X.for) == 1){
+      if(is.null(X.for) == TRUE){
+        M = 50
+      }
+      else{
+        M = length(X.for)
+      }
+    }
+    else{
+      M = length(X.for[,1])
+    }
+  }
+  else if(missing(X.for) == FALSE && missing(M) == FALSE){
+    if (NCOL(X.for) == 1){
+      if(is.null(X.for) == TRUE){
+        M = M
+      }
+      else{
+        M = min(length(X.for), M)
+      }
+    }
+    else{
+      M = min(length(X.for[,1]),M)
+    }
+  }
+
+  coef.caus <- c()
+  if (object$order[1] == 0){
+    r = 1
+    coef.caus <- object$coefficients[(r+1)]
+  }
+  else{
+    r = object$order[1]
+    coef.caus <- object$coefficients[2:(r+1)]
+  }
+
+  coef.noncaus <- c()
+  if (object$order[2] == 0){
+    s = 1
+    coef.noncaus <- object$coefficients[(r+1+s)]
+  }
+  else{
+    s = object$order[2]
+    coef.noncaus <- object$coefficients[(r+2):(r+1+s)]
+  }
+
+  coef.exo <- c()
+  if (object$order[3] == 0){
+    q = 1
+    coef.exo <- object$coefficients[(r+1+s+q)]
+  }
+  else{
+    q = object$order[3]
+    coef.exo <- object$coefficients[(r+1+s+1):(r+s+1+q)]
+  }
+
+  ## Simulate future epsilon and use forecasted X
+  hve <- c()
+  hve2 <- matrix(data=0, nrow=N,ncol=h)
+
+  for (iter in 1:N){
+
+    eps.sim <- object$coefficients["scale",]*stats::rt(M,object$coefficients["df",])
+
+    z2 <- c()
+    for (i in 1:M){
+      if(is.null(X.for) == TRUE){
+        z2[i] <- eps.sim[i]
+      }
+      else{
+        if(NCOL(X.for) > 1){
+          z2[i] <- eps.sim[i] +  coef.exo %*% t(X.for[i,])
+        }
+        else{
+          z2[i] <- eps.sim[i] + coef.exo * X.for[i]
+        }
+      }
+    }
+
+    ## Compute filtered values u = phi(L)y and moving average values
+    phi <- c(1,coef.caus)
+
+    u <- c()
+    for (i in (r+1):obs){
+      u[i] <- phi %*% y[i:(i-r)]
+    }
+    w <- c(u[(obs-s+1):obs],z2)
+
+    C <- matrix(data=0, nrow=(M+s), ncol=(M+s))
+    C[1,] <- compute.MA(coef.noncaus,(M+s-1))
+
+    if (s > 1){
+      for (i in 2:s){
+        C[i,] <- c(0, C[(i-1),1:(length(C[(i-1),])-1)])
+      }
+    }
+
+    for (i in (s+1):(M+s)){
+      C[i,] <- c(rep(0,(i-1)),1,rep(0,(M+s-i)))
+    }
+
+    D = solve(C)
+
+    e <- D %*% w
+
+    h1 <- c()
+
+    for (i in 1:s){
+      h1[i] <- metRology::dt.scaled(e[i], df=object$coefficients["df",], sd=object$coefficients["scale",])
+    }
+
+    hve[iter] = prod(h1)
+
+    for (j in 1:h){
+      mov.av <-  C[1,1:(M-j+1)] %*% z2[j:M]
+      hve2[iter,j] <- mov.av * hve[iter]
+
+    }
+  }
+
+  y.star <- y[(obs-r+1):obs]
+  y.for <- c()
+  exp <- c()
+
+  for (j in 1:h){
+    exp[j] = ((1/N)*sum(hve2[,j]))/((1/N)*sum(hve))
+
+    if(length(coef.caus) == 1){
+      y.for[j] <-  object$coefficients[1]/(1-sum(coef.noncaus)) + coef.caus * y.star + exp[j]
+    }
+    else{
+      y.for[j] <-  object$coefficients[1]/(1-sum(coef.noncaus)) + t(coef.caus) %*% y.star + exp[j]
+    }
+
+    y.star <- c(y.for[j], y.star[1:(length(y.star)-1)])
+  }
+
+  return(y.for)
+}
+
+mixed.combine <- function(y,x,p_C,p_NC){
+
+  q <- NCOL(x)
+
+  if (is.null(x) == TRUE){
+    x <- "not"
+    q <- 0
+  }
+
+  sig_level = 0.05
+  est <- marx.t(y,x,p_C,p_NC)
+  residuals <- est$residuals
+  fitted.values <- y[(1+p_C):(length(y)-p_NC)] - residuals
+
+  inf <- inference(y,x,est$coef.c,est$coef.nc,est$coef.exo,est$coef.int,est$scale,est$df,sig_level)
+
+  coefficients <- c(est$coef.int,est$coef.c,est$coef.nc,est$coef.exo, est$df, est$scale)
+  coefficients <- fBasics::vec(coefficients)
+
+  if(length(x) > 1){
+    if (p_C > 0 && p_NC > 0){
+      rownames(coefficients) <- c('int', paste('lag', 1:p_C), paste('lead', 1:p_NC), paste('exo', 1:NCOL(x)), 'df', 'scale')
+    }
+    else if (p_C > 0 && p_NC == 0){
+      rownames(coefficients) <- c('int', paste('lag', 1:p_C), 'lead', paste('exo', 1:NCOL(x)), 'df', 'scale')
+    }
+    else if (p_C == 0 && p_NC > 0){
+      rownames(coefficients) <- c('int', 'lag', paste('lead', 1:p_NC), paste('exo', 1:NCOL(x)), 'df', 'scale')
+    }
+    else if (p_C == 0 && p_NC == 0){
+      rownames(coefficients) <- c('int', 'lag', 'lead', paste('exo', 1:NCOL(x)), 'df', 'scale')
+    }
+  }
+  else{
+    if (p_C > 0 && p_NC > 0){
+      rownames(coefficients) <- c('int', paste('lag', 1:p_C), paste('lead', 1:p_NC), 'exo', 'df', 'scale')
+    }
+    else if (p_C > 0 && p_NC == 0){
+      rownames(coefficients) <- c('int', paste('lag', 1:p_C), 'lead', 'exo', 'df', 'scale')
+    }
+    else if (p_C == 0 && p_NC > 0){
+      rownames(coefficients) <- c('int', 'lag', paste('lead', 1:p_NC), 'exo', 'df', 'scale')
+    }
+    else if (p_C == 0 && p_NC == 0){
+      rownames(coefficients) <- c('int', 'lag', 'lead', 'exo', 'df', 'scale')
+    }
+  }
+
+  se <- c(inf$se.int, inf$se.c, inf$se.nc, inf$se.exo, est$se.dist[1], est$se.dist[2])
+
+  if (length(x) > 1){
+    degree <- NROW(est$residuals) - (p_C + p_NC + NCOL(x) + 1)
+  }
+  else{
+    degree <- NROW(est$residuals) - (p_C + p_NC + 1)
+  }
+
+  return(list(coefficients = coefficients, se = se, df.residual = degree, residuals= residuals, fitted.values = fitted.values, order=c(p_C,p_NC,q)))
+}
+
+
+#' @title  The pseudo-causal model function
+#' @description This function allows you to estimate pseudo-causal ARX models by OLS (compatible with most functions in lm() class).
 #' @aliases   pseudo
 #'            pseudo.default
 #'            print.pseudo
@@ -1355,7 +1647,7 @@ sim.marx <- function(dist.eps, dist.x, obs,c_par,nc_par, exo_par){
 #' summary(object)
 #' @rdname pseudo
 #' @export pseudo
-   pseudo <- function(y,x,p){ UseMethod("pseudo")}
+pseudo <- function(y,x,p){ UseMethod("pseudo")}
 #' @return An object of class \code{"pseudo"} is a list containing the following components:
 #' @return \item{coefficients}{Vector of estimated coefficients.}
 #' @return \item{coef.auto}{Vector of estimated autoregressive parameters.}
@@ -1369,51 +1661,49 @@ sim.marx <- function(dist.eps, dist.x, obs,c_par,nc_par, exo_par){
 #'
 #' @rdname pseudo
 #' @method pseudo default
-#' @S3method pseudo default
-   pseudo.default <- function(y,x,p){
+#' @S3method  pseudo default
+pseudo.default <- function(y,x,p){
 
-     if (is.null(x)){
-       x <- "not"
-     }
+  if (is.null(x)){
+    x <- "not"
+  }
 
-     est <- arx.ls(y,x,p)
-     est$call <- match.call();
-     class(est) <- "pseudo"
-     est
-   }
-#'
+  est <- arx.ls(y,x,p)
+  est$call <- match.call();
+  class(est) <- "pseudo"
+  est
+}
 #' @rdname pseudo
 #' @method print pseudo
 #' @S3method print pseudo
-   print.pseudo <- function(x,...){
+print.pseudo <- function(x,...){
 
-     cat("Call:\n")
-     print(x$call)
-     cat("\n Coefficients:\n")
-     print(x$coefficients)
-   }
-#'
+  cat("Call:\n")
+  print(x$call)
+  cat("\n Coefficients:\n")
+  print(x$coefficients)
+}
 #' @rdname pseudo
 #' @method summary pseudo
 #' @S3method summary pseudo
-   summary.pseudo <- function(object,...){
+summary.pseudo <- function(object,...){
 
-     se <- sqrt(diag(object$vcov))
-     tval <- stats::coef(object) / se
+  se <- sqrt(diag(object$vcov))
+  tval <- stats::coef(object) / se
 
-     TAB <- cbind(Estimate = stats::coef(object),
-                  StdErr = se,
-                  t.value = tval,
-                  p.value = 2*stats::pt(-abs(tval),df=object$df))
+  TAB <- cbind(Estimate = stats::coef(object),
+               StdErr = se,
+               t.value = tval,
+               p.value = 2*stats::pt(-abs(tval),df=object$df))
 
-     colnames(TAB) <- c("Estimate", "Std.Err", "t value", "p value")
+  colnames(TAB) <- c("Estimate", "Std.Err", "t value", "p value")
 
-     res <- list(call=object$call, coefficients = TAB)
+  res <- list(call=object$call, coefficients = TAB)
+
+  class(res) <- "summary.pseudo"
+  res
+}
 
-     class(res) <- "summary.pseudo"
-     res
-   }
-#'
 
 pseudo <- function(y,x,p){ UseMethod("pseudo")}
 
@@ -1445,7 +1735,7 @@ summary.pseudo <- function(object,...){
   tval <- stats::coef(object) / se
 
   TAB <- cbind(Estimate = stats::coef(object),
-               StdErr =  se,
+               StdErr = se,
                t.value = tval,
                p.value = 2*stats::pt(-abs(tval),df=object$df))
 
@@ -1457,12 +1747,10 @@ summary.pseudo <- function(object,...){
   res
 }
 
-#' The MARX estimation function
-#'
-#' This function allows you to estimate mixed causal-noncausal MARX models by t-MLE (compatible with most functions in lm() class).
+#' @title The MARX estimation function
+#' @description This function allows you to estimate mixed causal-noncausal MARX models by t-MLE (compatible with most functions in lm() class).
 #' @aliases    mixed
 #'             mixed.default
-#'             mixed.combine
 #'             print.mixed
 #'             summary.mixed
 #' @param y      Data vector of time series observations.
@@ -1480,197 +1768,64 @@ summary.pseudo <- function(object,...){
 #' summary(object)
 #' @rdname mixed
 #' @export mixed
-   mixed <- function(y,x,p_C,p_NC){ UseMethod("mixed")}
+mixed <- function(y,x,p_C,p_NC){ UseMethod("mixed")}
 #' @return An object of class \code{"mixed"} is a list containing the following components:
 #' @return \item{coefficients}{Vector of estimated coefficients.}
 #' @return \item{se}{Standard errors of estimated coefficients.}
 #' @return \item{df.residual}{Degrees of freedom residuals.}
 #' @return \item{residuals}{Residuals.}
 #' @return \item{fitted.values}{Fitted values.}
+#' @return \item{order}{Vector containing (r,s,q), i.e. causal order r, noncausal order s, number of exogenous regressors q.}
 #' @rdname mixed
 #' @method mixed default
 #' @S3method mixed default
-   mixed.default <- function(y,x,p_C,p_NC){
+mixed.default <- function(y,x,p_C,p_NC){
 
-     if (is.null(x)){
-       x <- "not"
-     }
-
-     vals <- mixed.combine(y,x,p_C,p_NC);
-     vals$call <- match.call();
-     class(vals) <- "mixed"
-     vals
-   }
-#'
+  est <- mixed.combine(y,x,p_C,p_NC);
+  est$call <- match.call();
+  class(est) <- "mixed"
+  est
+}
 #' @rdname mixed
 #' @method print mixed
 #' @s3method print mixed
-   print.mixed <- function(x,...){
-     cat("Call:\n")
-     print(x$call)
-     cat("\n Coefficients:\n")
-     print(x$coefficients)
-   }
-#'
+print.mixed <- function(x,...){
+
+  cat("Call:\n")
+  print(x$call)
+  cat("\n Coefficients:\n")
+  print(x$coefficients)
+}
 #' @rdname mixed
 #' @method summary mixed
-#' @s3method summary mixed
-   summary.mixed <- function(object,...){
+#' @S3method summary mixed
+summary.mixed <- function(object,...){
 
-     se <- sqrt(diag(object$vcov))
-     tval <- stats::coef(object) / se
+  tval <- stats::coef(object) / object$se
 
-     TAB <- cbind(Estimate = stats::coef(object),
-                  StdErr = object$se,
-                  t.value = tval,
-                  p.value = 2*stats::pt(-abs(tval),df=object$df.residual))
+  TAB <- cbind(Estimate = stats::coef(object),
+               StdErr = object$se,
+               t.value = tval,
+               p.value = 2*stats::pt(-abs(tval),df=object$df.residual))
 
-     colnames(TAB) <- c("Estimate", "Std.Err", "t value", "p value")
+  colnames(TAB) <- c("Estimate", "Std.Err", "t value", "p value")
 
-     res <- list(call=object$call, coefficients = TAB)
-
-     class(res) <- "summary.mixed"
-     res
-   }
-#'
-#'
-#' @rdname mixed
-#' @method mixed combine
-#' @S3method mixed combine
-   mixed.combine <- function(y,x,p_C,p_NC){
-
-     if (is.null(x)){
-       x <- "not"
-     }
-
-     sig_level = 0.05
-     est <- marx.t(y,x,p_C,p_NC)
-     residuals <- est$residuals
-     fitted.values <- y[(1+p_C):(length(y)-p_NC)] - residuals
-
-     inf <- inference(y,x,est$coef.c,est$coef.nc,est$coef.exo,est$coef.int,est$scale,est$df,sig_level)
-
-     coefficients <- c(est$coef.int,est$coef.c,est$coef.nc,est$coef.exo)
-     coefficients <- fBasics::vec(coefficients)
-
-     if(length(x) > 1){
-       if (p_C > 0 && p_NC > 0){
-         rownames(coefficients) <- c('int', paste('lag', 1:p_C), paste('lead', 1:p_NC), paste('exo', 1:NCOL(x)))
-       }
-       else if (p_C > 0 && p_NC == 0){
-         rownames(coefficients) <- c('int', paste('lag', 1:p_C), 'lead', paste('exo', 1:NCOL(x)))
-       }
-       else if (p_C == 0 && p_NC > 0){
-         rownames(coefficients) <- c('int', 'lag', paste('lead', 1:p_NC), paste('exo', 1:NCOL(x)))
-       }
-       else if (p_C == 0 && p_NC == 0){
-         rownames(coefficients) <- c('int', 'lag', 'lead', paste('exo', 1:NCOL(x)))
-       }
-     }
-     else{
-       if (p_C > 0 && p_NC > 0){
-         rownames(coefficients) <- c('int', paste('lag', 1:p_C), paste('lead', 1:p_NC), 'exo')
-       }
-       else if (p_C > 0 && p_NC == 0){
-         rownames(coefficients) <- c('int', paste('lag', 1:p_C), 'lead', 'exo')
-       }
-       else if (p_C == 0 && p_NC > 0){
-         rownames(coefficients) <- c('int', 'lag', paste('lead', 1:p_NC), 'exo')
-       }
-       else if (p_C == 0 && p_NC == 0){
-         rownames(coefficients) <- c('int', 'lag', 'lead', 'exo')
-       }
-     }
-
-     se <- c(inf$se.int, inf$se.c, inf$se.nc, inf$se.exo)
-
-     if (length(x) > 1){
-       degree <- NROW(est$residuals) - (p_C + p_NC + NCOL(x) + 1)
-     }
-     else{
-       degree <- NROW(est$residuals) - (p_C + p_NC + 1)
-     }
-
-     return(list(coefficients = coefficients, se = se, df.residual = degree, residuals= residuals, fitted.values = fitted.values))
-   }
-#'
+  res <- list(call=object$call, coefficients = TAB)
 
+  class(res) <- "summary.mixed"
+  res
+}
 
 mixed <- function(y,x,p_C,p_NC){ UseMethod("mixed")}
 
 mixed.default <- function(y,x,p_C,p_NC){
 
-  if (is.null(x)){
-    x <- "not"
-  }
-
-  vals <- mixed.combine(y,x,p_C,p_NC);
-  vals$call <- match.call();
-
-  class(vals) <- "mixed"
-  vals
+  est <- mixed.combine(y,x,p_C,p_NC);
+  est$call <- match.call();
+  class(est) <- "mixed"
+  est
 }
 
-
-mixed.combine <- function(y,x,p_C,p_NC){
-
-  if (is.null(x)){
-    x <- "not"
-  }
-
-  sig_level = 0.05
-  est <- marx.t(y,x,p_C,p_NC)
-  residuals <- est$residuals
-  fitted.values <- y[(1+p_C):(length(y)-p_NC)] - residuals
-
-  inf <- inference(y,x,est$coef.c,est$coef.nc,est$coef.exo,est$coef.int,est$scale,est$df,sig_level)
-
-  coefficients <- c(est$coef.int,est$coef.c,est$coef.nc,est$coef.exo)
-  coefficients <- fBasics::vec(coefficients)
-
-  if(length(x) > 1){
-    if (p_C > 0 && p_NC > 0){
-      rownames(coefficients) <- c('int', paste('lag', 1:p_C), paste('lead', 1:p_NC), paste('exo', 1:NCOL(x)))
-    }
-    else if (p_C > 0 && p_NC == 0){
-      rownames(coefficients) <- c('int', paste('lag', 1:p_C), 'lead', paste('exo', 1:NCOL(x)))
-    }
-    else if (p_C == 0 && p_NC > 0){
-      rownames(coefficients) <- c('int', 'lag', paste('lead', 1:p_NC), paste('exo', 1:NCOL(x)))
-    }
-    else if (p_C == 0 && p_NC == 0){
-      rownames(coefficients) <- c('int', 'lag', 'lead', paste('exo', 1:NCOL(x)))
-    }
-  }
-  else{
-    if (p_C > 0 && p_NC > 0){
-      rownames(coefficients) <- c('int', paste('lag', 1:p_C), paste('lead', 1:p_NC), 'exo')
-    }
-    else if (p_C > 0 && p_NC == 0){
-      rownames(coefficients) <- c('int', paste('lag', 1:p_C), 'lead', 'exo')
-    }
-    else if (p_C == 0 && p_NC > 0){
-      rownames(coefficients) <- c('int', 'lag', paste('lead', 1:p_NC), 'exo')
-    }
-    else if (p_C == 0 && p_NC == 0){
-      rownames(coefficients) <- c('int', 'lag', 'lead', 'exo')
-    }
-  }
-
-  se <- c(inf$se.int, inf$se.c, inf$se.nc, inf$se.exo)
-
-  if (length(x) > 1){
-    degree <- NROW(est$residuals) - (p_C + p_NC + NCOL(x) + 1)
-  }
-  else{
-    degree <- NROW(est$residuals) - (p_C + p_NC + 1)
-  }
-
-  return(list(coefficients = coefficients, se = se, df.residual = degree, residuals= residuals, fitted.values = fitted.values))
-}
-
-
-
 print.mixed <- function(x,...){
 
   cat("Call:\n")
@@ -1696,5 +1851,3 @@ summary.mixed <- function(object,...){
   class(res) <- "summary.mixed"
   res
 }
-
-
